@@ -4,8 +4,8 @@ use warnings;
 use strict;
 use Carp;
 
-use version; our $VERSION = qv('0.9.1');
-
+use version; our $VERSION = qv('0.9.2');
+use Data::Dumper;
 # Input files are assumed to be in the UTF-8 strict character encoding.
 use utf8;
 binmode(STDOUT, ":utf8");
@@ -27,16 +27,17 @@ sub new {
     bless $self, $class;
 
     # Get the Configurations for building these regular expressions
-    my %bookconfig = $self->_process_config($config);
+    my %configs;
+    $self->_process_config($config->get_search_configurations, \%configs);
 
-    #################################################################################### 
+    ######################################################################################
     #	Définitions par défaut des expressions régulières avec références bibliques
     #  
     # 	La fonction '_set_regex' a trois paramètres.
     #		1. Un nom unique pour cette expression régulière
     #		2. Une experssion régulière
     #		3. Si la paramètre deux est 'undef', une experssion régulière comme defaut 
-    #################################################################################### 
+    ###################################################################################### 
     
     my $spaces = qr/([\s ]*)/;
 
@@ -46,21 +47,21 @@ sub new {
     # chapitre : c'est un nombre qui indique un chapitre
     my $chapitre = qr/\d{1,3}/;
     $self->_set_regex(	'chapitre', 
-			$bookconfig{'chapitre'}, 
+			$configs{'chapitre'}, 
             $chapitre
 		    );
 
     # verset : c'est un nombre qui indique un verset
     my $verset = qr/\d{1,3}[abcdes]?/;
     $self->_set_regex(	'verset', 
-			$bookconfig{'verset'}, 
+			$configs{'verset'}, 
 			$verset
 		    );
 
     # chiffre : c'est un nombre qui indique un chapitre ou verset 
     my $chiffre = qr/\d{1,3}[abcdes]?/;
     $self->_set_regex(	'chiffre', 
-			$bookconfig{'chiffre'}, 
+			$configs{'chiffre'}, 
 			$chiffre
 		    );
 
@@ -71,43 +72,44 @@ sub new {
     # cv_separateur : vous pouvez trouver ce entre un chapitre et un verset
     my $cv_separateur = qr/(?::|\.)/;
     $self->_set_regex(	'cv_separateur', 
-			$bookconfig{'cv_separateur'}, 
+			$configs{'cv_separateur'}, 
 			$cv_separateur	
 		    );
 
     # separateur : cette sépare deux références bibliques
     my $separateur = qr/\bet\b/;
     $self->_set_regex(	'separateur', 
-			$bookconfig{'separateur'}, 
+			$configs{'separateur'}, 
 			$separateur	
 		    );
 
     # cl_separateur : cette sépare deux références bibliques et que le deuxième référence est un référence d'un chaptire
     my $cl_separateur = qr/;/;
     $self->_set_regex(	'cl_separateur', 
-			$bookconfig{'cl_separateur'}, 
+			$configs{'cl_separateur'}, 
 			$cl_separateur	
 		    );
 
     # vl_separateur : cette sépare deux références bibliques et que le deuxième référence est un référence d'un verset
     my $vl_separateur = qr/,/;
     $self->_set_regex(	'vl_separateur', 
-			$bookconfig{'vl_separateur'}, 
+			$configs{'vl_separateur'}, 
 			$vl_separateur		
 		    );
 
     my $intervale = qr/(?:-|–|−)/;
     # tiret : ce correspond à tous les types de tiret
     $self->_set_regex(	'intervale', 
-			$bookconfig{'intervale'}, 
+			$configs{'intervale'}, 
 			$intervale	
 		    );
+
     # reference_separateurs : ce correspond à tous les types de separateur entre références biblque 
     my $cl_ou_vl_separateur = qr/(?:$self->{cl_separateur}|$self->{vl_separateur}|$self->{separateur})/;
     $self->_set_regex(	'cl_ou_vl_separateurs', 
-			$bookconfig{'cl_ou_vl_separateurs'}, 
+			$configs{'cl_ou_vl_separateurs'}, 
 			$cl_ou_vl_separateur	
-		    );
+                     );
 
 
     #################################################################################### 
@@ -125,8 +127,8 @@ sub new {
     # intervale_chiffre : deux chapitre avec un tiret entre
     # Par exemple: '-2', '–9', ou ' - 4'
     $self->_set_regex(	'intervale_chiffre', 
-			$bookconfig{'intervale_chiffre'},  
-		    $intervale_chiffre	
+			$configs{'intervale_chiffre'},  
+                        $intervale_chiffre	
 		    );
 
     my $cv_separateur_chiffre = qr/
@@ -140,52 +142,53 @@ sub new {
     # cv_separateur_chiffre : deux chapitre avec un tiret entre
     # Par exemple: ':2', '.9', ou ' : 4'
     $self->_set_regex(	'cv_separateur_chiffre', 
-			$bookconfig{'cv_separateur_chiffre'}, 
-		    $cv_separateur_chiffre	
-		    );
+			$configs{'cv_separateur_chiffre'}, 
+                        $cv_separateur_chiffre	
+		     );
 
     #################################################################################### 
     # Définitions de les references numiques 
     #################################################################################### 
 
-    #######################################################################################################
+    ####################################################################################
     # Les mots donne contexte aux référence biblique
     # Par Exemple: 
     #   chapitre_mots: 'voir la chapitre'
     #   texte: voir la chapitre 9
     # 
-    #   Avec cette texte 'voir la chapitre' comme chapitre_mots le 9 peu être indentifié comme un chapitre
-    #######################################################################################################
+    #   Avec cette texte 'voir la chapitre' comme chapitre_mots le 9 peu être indentifié 
+    #   comme un chapitre
+    #####################################################################################
    
-    # reference_contexte_mots_avant : les mots qui indique que le prochain référence est un chapitre référence
+    # reference_contexte_mots_avant : les mots qui indique que le prochain référence
+    #                                 est un chapitre référence
     my $reference_mots = qr/(?:dans|voir aussi)/;
     $self->_set_regex(	'reference_mots', 
-			$bookconfig{'reference_mots'}, 
-            $reference_mots			
+			$configs{'reference_mots'}, 
+                        $reference_mots			
     );
 
     # chapitre_contexte_mots_avant : les mots qui indique que le prochain référence est un chapitre référence
     my $chapitre_mots = qr/(?:dans le chapitre)/;
     $self->_set_regex(	'chapitre_mots', 
-			$bookconfig{'chapitre_mots'}, 
-            $chapitre_mots			
+			$configs{'chapitre_mots'}, 
+                        $chapitre_mots			
     );
 
     # verset_contexte_mots_avant : les mots qui indique que le prochain référence est un verset référence
     my $verset_mots = qr/(?:vv?\.)/;
     $self->_set_regex(	'verset_mots', 
-			$bookconfig{'verset_mots'},  
-            $verset_mots
+			$configs{'verset_mots'},  
+                        $verset_mots
     );
 
     # voir_contexte_mots_avant : les mots qui indique que le prochain référence est un verset référence
     my $voir_mots = qr/(?:voir)/;
     $self->_set_regex(	'voir_mots', 
-			$bookconfig{'voir_mots'},  
-            $voir_mots
+			$configs{'voir_mots'},  
+                        $voir_mots
     );
 
-
     #################################################################################### 
     # Définitions de les expressions avec livres 
     #################################################################################### 
@@ -196,8 +199,8 @@ sub new {
     /x;
 
     $self->_set_regex(	'livres_numerique', 
-			$bookconfig{'livres_numerique'}, 
-            $livres_numerique
+			$configs{'livres_numerique'}, 
+                        $livres_numerique
     );
 
     my $livres_numerique_protect = "";
@@ -205,7 +208,7 @@ sub new {
         $livres_numerique_protect = qr/(?!(?:[\s ]*(?:$livres_numerique)))/;
     }
     $self->_set_regex(   'livres_numerique_protect',
-            $bookconfig{'livres_numerique_protect'},
+            $configs{'livres_numerique_protect'},
             $livres_numerique_protect
     );
 
@@ -216,8 +219,8 @@ sub new {
 
     # livres : le nom complet de tous les livres, avec et sans accents
     $self->_set_regex(	'livres', 
-			$bookconfig{'livres'}, 
-            $livres
+			$configs{'livres'}, 
+                        $livres
     );
 
     my $abbreviations = qr/
@@ -226,16 +229,16 @@ sub new {
     
     # abbreviations : le nom complet de tous les abbreviations, avec et sans accents
     $self->_set_regex(	'abbreviations', 
-			$bookconfig{'abbreviations'}, 
-            $abbreviations
-    );
+			$configs{'abbreviations'}, 
+                        $abbreviations
+	);
 
     # livres_et_abbreviations : la liste de tous les livres et les abréviations
     my $livres_et_abbreviations = qr/(?:$self->{'livres'}|$self->{'abbreviations'})/;
     $self->_set_regex(	'livres_et_abbreviations', 
-			$bookconfig{'livres_et_abbreviations'}, 
-            $livres_et_abbreviations
-		    );
+			$configs{'livres_et_abbreviations'}, 
+                        $livres_et_abbreviations
+	);
 
     # contexte_mots : Tous les mots qui viennent avant une référence biblique. Des mots différents peut 
     #                fournir des contextes différents. Par exemple, 'voir le chapitre' fournit une 
@@ -253,41 +256,41 @@ sub new {
     /x;
 
     $self->_set_regex(	'contexte_mots', 
-			$bookconfig{'contexte_mots'}, 
-            $contexte_mots
+			$configs{'contexte_mots'}, 
+			$contexte_mots
 		    );
 
-    #livre2abre : une table de changement du livre à l'abréviation
+    #livre2abre : une table de changement pour livre à l'abréviation
     $self->_set_hash(	'book2key', 
-			$bookconfig{'book2key'}, 
-            {}
+			$configs{'book2key'}, 
+                        {}
     );
     
-    #abre2livres : une table de changement du abréviation à livre
+    #abre2livres : une table de changement pour abréviation à livre
     $self->_set_hash(	'abbr2key', 
-			$bookconfig{'abbr2key'}, 
+			$configs{'abbr2key'}, 
 			{}
 		    );
 
-    #livre2abre : une table de changement du livre à l'abréviation
+    #livre2abre : une table de changement pour livre à l'abréviation
     $self->_set_hash(	'key2book', 
-			$bookconfig{'key2book'}, 
-            {}
+			$configs{'key2book'}, 
+                        {}
     );
     
-    #abre2livres : une table de changement du abréviation à livre
+    #abre2livres : une table de changement pour abréviation à livre
     $self->_set_hash(	'key2abbr', 
-			$bookconfig{'key2abbr'}, 
+			$configs{'key2abbr'}, 
 			{}
 		    );
 
 
     # livres_avec_un_chapitre :  la liste de tous les livres avec un seul chapitre
-    my $livres_avec_un_chapitre = qr/Ab|Abdias|2Jn|2Jean|Phm|Philemon|Philémon|Jud|Jude|3Jn|3Jean/;
+    my $livres_avec_un_chapitre = qr/(?:Ab|Abdias|2Jn|2Jean|Phm|Philemon|Philémon|Jud|Jude|3Jn|3Jean)/;
     $self->_set_regex(	'livres_avec_un_chapitre', 
-			$bookconfig{'livres_avec_un_chapitre'}, 
-            $livres_avec_un_chapitre
-		    );
+			$configs{'livres_avec_un_chapitre'}, 
+                        $livres_avec_un_chapitre
+	             );
 
     #######################################################################################################
     # full_reference_protection : Il s'agit d'une expression régulière complexe. Ne pas changer, 
@@ -323,9 +326,9 @@ sub new {
 
     # cv_list : Combines LC, LCC, LCCV, LCCVV and LCV, LCVV, LCVCV
     $self->_set_regex(	'cv_list', 
-			$bookconfig{'cv_list'},
-	        $cv_list	
-		    );
+			$configs{'cv_list'},
+	                $cv_list	
+		     );
 
 
     # reference_biblique_list : Cette expression régulière correspond à une liste de références bibliques 
@@ -350,9 +353,9 @@ sub new {
     /x;
 
     $self->_set_regex(	'reference_biblique', 
-			$bookconfig{'reference_biblique'}, 
-		    $reference_biblique
-		    );
+			$configs{'reference_biblique'}, 
+                        $reference_biblique
+		     );
 
     # reference_biblique_list : Cette expression régulière correspond à une liste de références bibliques 
     #				ex. '1 Ti 1.19 ; Ge 1:1, 2:16-18' or '1 Ti 1.19 ; 2Ti 2:16-18'
@@ -377,7 +380,7 @@ sub new {
     /x;
 
     $self->_set_regex(	'reference_biblique_list', 
-			$bookconfig{'reference_biblique_list'}, 
+			$configs{'reference_biblique_list'}, 
 		    $reference_biblique_list
 		    );
 
@@ -390,6 +393,7 @@ sub new {
 ################################################################################
 sub _set_regex {
     my ($self, $key, $regex, $default_regex) = @_;
+#    return '' if (m/^$/ =~ $regex);
 	if (defined($regex)) {
         my $result = qr/$regex/;	        # Evaluate that line
         if ($@) {                       	# Check for compile or run-time errors.
@@ -411,7 +415,6 @@ sub _set_hash {
     } else {
         $self->{$key} = $default_hash;
     }
-
 }
 
 ################################################################################
@@ -420,20 +423,26 @@ sub _set_hash {
 sub _process_config {
     my $self = shift;
     my $config = shift;
-    my %retval;
+    my $retval = shift;
 
-    while ( my ($key, $value) = each(%{$config->{config}{regex}}) ) {
-        if ($key =~ m/definitions/) {
-            $self->_init_data_structures($value, $config, \%retval);
+    # If this is the book configurations then build the associated data structures
+    # If this configuration value is a file name, then use the contents of that
+    #    file to build a regular expression
+    # If the configuration value is a HASH, then recursively call  _process_config
+    # If this configuration value is a string, then copy it to the data structure
+    #    that is being returned
+    while ( my ($key, $value) = each(%{$config}) ) {
+        if ($key =~ m/books/) {
+            $self->_init_book_and_abbreviation_data_structures($value, $retval);
         } elsif ($value =~ m/^(?:fichier|file):/) {
-            $retval{$key} = $self->_build_regexes_from_file($value);
+            $retval->{$key} = $self->_build_regexes_from_file($value);
         } elsif (defined(ref $value) && ref $value eq "HASH") {
-            $retval{$key} = $self->_process_config($value);
+            $self->_process_config($value, $retval);
         } else {
-            $retval{$key} = $value;    
+            $retval->{$key} = $value;    
         }
     }
-    return %retval;
+    return $retval;
 }
 
 sub _build_regexes_from_file {
@@ -454,7 +463,7 @@ sub _build_regexes_from_file {
         push @list, $_;
     }
     close (LIST);
-    return "(?:" . join_regex(\@list) . ")";
+    return "(?:" . _join_regex(\@list) . ")";
 }
 
 sub _join_regex {
@@ -468,7 +477,7 @@ sub _join_regex {
 
 
 ################################################################################
-# _init_data_structures
+# _init_book_and_abbreviation_data_structures
 # 
 # Creates the following mappings:
 #   An array of all match book names (book names to search for in a document)
@@ -488,19 +497,17 @@ sub _join_regex {
 #        Abbreviation: Ge
 #
 ################################################################################
-sub _init_data_structures {
+sub _init_book_and_abbreviation_data_structures {
     my $self = shift;
-    my $path_to_config_file = shift;
     my $config = shift;
     my $retval = shift;
 
-    my $bookconfig = $config->{bookconfig};
     my $regex;
     my (@livres, @livres_numerique, @abbreviations);    # Array for all match books and another for match books starting with a number
     my (%book2key, %abbr2key, %key2abbr, %key2book);    # Mappings between match books and abbreviations and the primary key
     
     # Loop through each number and gather the books
-    while( my ($key, $value) = each %{$bookconfig} ) {
+    while ( my ($key, $value) = each %{$config} ) {
         # Loop through 
         foreach my $livre (@{$value->{Match}{Book}}) {
             push @livres, $livre;
@@ -516,9 +523,9 @@ sub _init_data_structures {
         $key2book{$key} = $value->{Normalized}{Book};
     }
      
-    $retval->{'livres'} = join_regex(\@livres);
-    $retval->{'abbreviations'} = join_regex(\@abbreviations);
-    $retval->{'livres_numerique'} = join_regex(\@livres_numerique);
+    $retval->{'livres'} = _join_regex(\@livres);
+    $retval->{'abbreviations'} = _join_regex(\@abbreviations);
+    $retval->{'livres_numerique'} = _join_regex(\@livres_numerique);
 
     $retval->{'livres_array'} = \@livres;
     $retval->{'abbreviations_array'} = \@abbreviations;
@@ -528,7 +535,7 @@ sub _init_data_structures {
     $retval->{'abbr2key'}   = \%abbr2key;
     $retval->{'key2book'}   = \%key2book;
     $retval->{'key2abbr'}   = \%key2abbr;
-    $retval->{'bookconfig'} = $bookconfig;
+    $retval->{'configs'} = $config;
 }
 
 
